"""Python utilities by the Professorship of Environmental
Sensing and Modeling at the Technical University of Munich"""

from . import (
    datastructures,
    decorators,
    files,
    github,
    logger,
    mathematics,
    processes,
    shell,
    system,
    testing,
    text,
    validators,
)
